import { reactive, ref } from "vue";
import { calcCursorLeft, SystemPivot } from "./cursor";
import { globalLookup } from "@/hook/global";
import formatTime from "@/hook/wave-view/format-time";

/**
 * @description 每一个动态信标
 * @typedef {Object} UserPivot
 * @property {number} id
 * @property {string} label
 * @property {number} x
 * @property {number} time
 * @property {boolean} show
 */


/**
 * @description 该数据结构描述了所有通过 makePivot 创建的 用户信标
 * @type {Map<number, UserPivot>}
 */
export const UserPivots = reactive(new Map());
export const orderedTimes = [];

export const UserPivotColor = ref('#1e90ff');

/**
 * @description 创建一个信标
 * @param {number} time 需要创建的轴的时间点
 */
export function createPivot(time) {
    if (UserPivots.has(time)) {
        return;
    }

    const id = Date.now();
    const x = calcCursorLeft(time);
    const timescale = globalLookup.timescale;
    const label = formatTime(time, timescale);
    const show = true;

    UserPivots.set(time, { time, x, id, label, show });
    orderedTimes.push(time);
    orderedTimes.sort((a, b) => a < b);
}

export function searchOrderedTimeIndex(time) {
    for (let i = 0; i < orderedTimes.length; ++ i) {
        if (orderedTimes[i] === time) {
            return i;
        }
    }
    return undefined;
}

export function deletePivot(time) {
    if (!UserPivots.has(time)) {
        return;
    }

    UserPivots.delete(time);
    const i = searchOrderedTimeIndex(time);
    if (i !== undefined) {
        orderedTimes.splice(i, 1);
    }
}

export function updateAllPivots() {
    // 更新系统信标
    SystemPivot.updateLeft();

    // 更新所有用户信标
    for (const time of UserPivots.keys()) {
        const pivot = UserPivots.get(time);
        pivot.x = calcCursorLeft(time);
    }
}

/**
 * 
 * @param {number[]} times 
 * @param {number} target 目标时间 
 * @returns 
 */
function bisearch(times, target) {
    let i = 0, j = times.length - 1;
    while (i < j) {
        if (times[i] === target) {
            break;
        }
        if (times[j] <= target) {
            i = j;
            break;
        }
        if (j - i === 1) {
            break;
        }
        const mid = (i + j) >> 1;
        if (times[mid] > target) {
            j = mid;
        } else {
            i = mid;
        }
    }

    return i;
}

/**
 * @description 高效获取距离 time 最近的时间点 t，时间复杂度 O(logn)
 * @param {number} time
 * @returns {UserPivot | undefined}
 */
export function getNearestUserPivot(time) {
    const pivotNum = UserPivots.size;
    if (pivotNum === 0) {
        return undefined;
    }

    if (pivotNum === 1) {
        for (const [time, pivot] of UserPivots) {
            return pivot;
        }
    }

    const i = bisearch(orderedTimes, time);
    // console.log(i, orderedTimes, time);
    
    if (i === pivotNum - 1) {
        return UserPivots.get(orderedTimes[i]);
    }
    
    const prevT = orderedTimes[i];
    const nextT = orderedTimes[i + 1];
    if (nextT - time > time - prevT) {
        return UserPivots.get(prevT);
    }
    return UserPivots.get(nextT);
}

/**
 * @type {Map<number, boolean>}
 */
export const UserPivotCtxShows = reactive(new Map());
